'''
clamav module, version 0.5

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>
                                                                                
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

'''

from avlib import *
import socket,re,string,tempfile

__all__=['clamscan','clamd','libclam']

def f_clamscan(stream,scanner):
    pf=popen(scanner+['-'])
    pf.write(fromhdr())
    pf.write(stream)
    pf.close(1)
    pf.wait()
    ret=[]
    for line in pf.readlines():
      if re.search(" FOUND",line):
        ret.append(line)
    pf.close(2)
    if ret==[]:
      vir=''
    else:
      l=re.search(": (.*) FOUND",string.rstrip(ret[0]))
      if l:
        vir=l.group(1)
      else:
        debug(0,'CLAMSCAN: ',ret)
        raise ascanner,'clamscan: '+str(ret)
    return vir,ret

def f_clamscandir(fda,scanner):
    ret=[]
    for fdir in fda:
      pf=popen(scanner+[fdir])
      pf.wait()
      for line in pf.readlines():
        if re.search(" FOUND",line):
          ret.append(line)
      pf.close()
    if ret==[]:
      vir=''
    else:
      l=re.search(": (.*) FOUND",string.rstrip(ret[0]))
      if l:
        vir=l.group(1)
      else:
        debug(0,'CLAMSCANF: ',ret)
        raise ascanner,'clamscandir: '+str(ret)
    return vir,ret

def f_clamscanf(data,scanner):
    mbox=mktemp('/tmp/clamscanner-','.sav.mbox').autorm()
    creatembox(mbox.f,data,1)
    return f_clamscandir([mbox.name],scanner)

def f_clamd(stream,scanner):
    if type(scanner)==type('string'): # local socket
      s=socket.socket(socket.AF_UNIX,socket.SOCK_STREAM)
      s.connect(scanner)
    else:
      s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
      s.connect((scanner[0],scanner[1]))
    f=s.makefile('rw',BUFSIZE)
    s.send("STREAM\n")
    rl=f.readline()
    l=re.search('^PORT ([0-9]*)',rl)
    if l:
      # Send stream
      ss=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
      if type(scanner)==type('string'): # local socket
        ss.connect(('127.0.0.1',string.atoi(l.group(1))))
      else:
        ss.connect((scanner[0],string.atoi(l.group(1))))
      ss.send(fromhdr())
      ss.send(stream)
      ss.shutdown(2)
      ss.close()
      # Read reply
      ret=[]
      for line in f.readlines():
        if not re.search(": OK$",string.rstrip(line)):
          ret.append(line)
      f.close()
      s.shutdown(2)
      s.close()
      if ret==[]:
        vir=''
      else:
        l=re.search(": (.*) FOUND$",string.rstrip(ret[0]))
        if l:
          vir=l.group(1)
        else:
          debug(0,'CLAMD: ',ret)
          raise ascanner,'clamd: '+str(ret)
      return vir,ret
    else:
      raise ascanner,'Protocol error, status: '+rl

class clamscan(ascanner):
  '''
  clamscan() realscanner - clamav scanner
  
  Usage: clamscan(['/usr/bin/clamscan','--stdout','--infected',
                   '--disable-summary','-r','--mbox'])

  Where: in [] are command line parameters for clamscan binary

  This scanners is a realscanner, which can be used to scan
  for viruses. ClamAV is a free program under GPL and may be freely
  used. Its database is very well.
  
  It can be easily used in chroot. makechroot from sagator package
  automatically integrates it into chroot environment.

  WARNING: If you are using clamscan from chroot, you need to configure
           freshclam to update chrooted copy of database!!!
  '''
  name='clamscan()'
  def __init__(self,arg):
      self.arg=arg
  def scanstream(self,stream):
      detected,virlist=f_clamscanf(stream,self.arg)
      return detected,virlist
  def scanfile(self,files,dirname=''):
      return f_clamscandir([dirname],self.arg)

class clamd(ascanner):
  '''
  clamd realscanner
  
  Usage: clamd(['localhost',3310])
     or: clamd('/var/spool/vscan/clamd')

  Where: 1st definition defines a clam daemon on host localhost,
             port 3310 (it is the default)
         2nd definition defines access to clamd via UNIX socket

  This scanners is a realscanner, which can be used to scan
  for viruses. ClamAV is a free program under GPL and may be freely
  used. Its database is very well.

  If you need a chrooted clamd, copy it into chroot and start it
  from here. If you are not familiar with it, use clamscan scanner
  from this module. It easily does chroot support, but it
  is not powerfull.
  '''
  name='clamd()'
  def __init__(self,arg):
      self.arg=arg
  def scanstream(self,stream):
      detected,virlist=f_clamd(stream,self.arg)
      return detected,virlist

try:
  import libclamav

  class libclam(ascanner):
    '''
    libclam realscanner - uses libclamavmodule python library
    
    Usage: libclam()

    This scanners is a realscanner, which can be used to scan
    for viruses. ClamAV is a free program under GPL and may be freely
    used. Its database is very well.
    
    WARNING: streamscanner can't scan mailboxes and archives.
             Use mimeparse() for mailboxes anb if you need also
             archive support, use decompress() interscanner.

    WARNING: If you are using clam from chroot, you need to configure
             freshclam to update chrooted copy of database!!!
    '''
    name='libclam()'
    def __init__(self):
        self.vpn=libclamav.loaddbdir()
        debug(3,"Loaded virpatterns: ",self.vpn)
    def reinit(self):
        debug(3,self.name,": Reloading database ...")
        libclamav.freetrie()
        self.vpn=libclamav.loaddbdir()
        debug(3,self.name,": Loaded virpatterns: ",self.vpn)
    def scanstream(self,stream):
        v=libclamav.scanbuff(stream)
        if v:
          return v,[v]
        else:
          return '',[]
    def scanfile(self,files,dir=''):
        for file in files:
          fd=os.open(os.path.join(dir,file),os.O_RDONLY)
          try:
            v=libclamav.scandesc(fd)
            os.close(fd)
            if v:
              return v,[v]
          except:
            os.close(fd)
            raise
          return '',[]
except ImportError:
  class libclam(ascanner):
    '''
    This method is undefined, because libclamavmodule is not found.
    Please recompile sagator, if you reaaly need it.
    '''
    name='libclam()'
    def __init__(self):
        pass
